@@ -1,5 +1,7 @@
 import sys
 
+import pytest
+
 from . import check, v
 
 assert v  # Silence pyflakes.
@@ -474,6 +476,18 @@ class Bar(object):
 def test_function_names_in_test_file(v):
     v.scan(
         """\
+def setup_module(module):
+    module
+
+def teardown_module(module):
+    module
+
+def setup_function(function):
+    function
+
+def teardown_function(function):
+    function
+
 def test_func():
     pass
 
@@ -481,7 +495,19 @@ def other_func():
     pass
 
 class TestClass:
-    pass
+    @classmethod
+    def setup_class(cls):
+        cls
+
+    @classmethod
+    def teardown_class(cls):
+        pass
+
+    def setup_method(self, method):
+        method
+
+    def teardown_method(self, method):
+        pass
 
 class BasicTestCase:
     pass
@@ -494,11 +520,26 @@ class OtherClass:
     check(v.defined_attrs, [])
     check(v.defined_classes, ["OtherClass"])
     check(v.defined_funcs, ["other_func"])
-    check(v.defined_vars, [])
-    check(v.used_names, [])
+    check(v.defined_methods, [])
+    check(
+        v.defined_vars,
+        [
+            "cls",
+            "cls",
+            "function",
+            "function",
+            "method",
+            "method",
+            "module",
+            "module",
+        ],
+    )
+    check(v.used_names, ["classmethod", "cls", "function", "method", "module"])
     check(v.unused_attrs, [])
     check(v.unused_classes, ["OtherClass"])
     check(v.unused_funcs, ["other_func"])
+    check(v.unused_methods, [])
+    check(v.unused_vars, [])
 
 
 def test_async_function_name_in_test_file(v):
@@ -741,3 +782,120 @@ bad()
         assert not v.found_dead_code_or_error
     else:
         assert v.found_dead_code_or_error
+
+
+def test_unused_args_with_del(v):
+    v.scan(
+        """\
+def foo(a, b, c, d=3):
+    del c, d
+    return a + b
+
+foo(1, 2)
+"""
+    )
+
+    check(v.defined_funcs, ["foo"])
+    check(v.defined_vars, ["a", "b", "c", "d"])
+    check(v.used_names, ["foo", "a", "b", "c", "d"])
+    check(v.unused_vars, [])
+    check(v.unused_funcs, [])
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 10), reason="requires python3.10 or higher"
+)
+def test_match_class_simple(v):
+    v.scan(
+        """\
+from dataclasses import dataclass
+
+
+@dataclass
+class X:
+    a: int
+    b: int
+    c: int
+    u: int
+
+x = input()
+
+match x:
+    case X(a=0):
+        print("a")
+    case X(b=0, c=0):
+        print("b c")
+"""
+    )
+    check(v.defined_classes, ["X"])
+    check(v.defined_vars, ["a", "b", "c", "u", "x"])
+
+    check(v.unused_classes, [])
+    check(v.unused_vars, ["u"])
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 10), reason="requires python3.10 or higher"
+)
+def test_match_class_embedded(v):
+    v.scan(
+        """\
+from dataclasses import dataclass
+
+
+@dataclass
+class X:
+    a: int
+    b: int
+    c: int
+    d: int
+    e: int
+    u: int
+
+x = input()
+
+match x:
+    case X(a=1) | X(b=0):
+        print("Or")
+    case [X(c=1), X(d=0)]:
+        print("Sequence")
+    case {"k": X(e=1)}:
+        print("Mapping")
+"""
+    )
+    check(v.defined_classes, ["X"])
+    check(v.defined_vars, ["a", "b", "c", "d", "e", "u", "x"])
+
+    check(v.unused_classes, [])
+    check(v.unused_vars, ["u"])
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 10), reason="requires python3.10 or higher"
+)
+def test_match_enum(v):
+    v.scan(
+        """\
+from enum import Enum
+
+
+class Color(Enum):
+    RED = 0
+    YELLOW = 1
+    GREEN = 2
+    BLUE = 3
+
+color = input()
+
+match color:
+    case Color.RED:
+        print("Real danger!")
+    case Color.YELLOW | Color.GREEN:
+        print("No danger!")
+"""
+    )
+    check(v.defined_classes, ["Color"])
+    check(v.defined_vars, ["RED", "YELLOW", "GREEN", "BLUE", "color"])
+
+    check(v.unused_classes, [])
+    check(v.unused_vars, ["BLUE"])
